 package ch.hepia.tpscala

/*
 * Implémenter les fonctions suivantes en suivant les commentaires.
 */

object Predicates {

  type P[A] = A=>Boolean

  /*
   * La méthode 'not' retourne un nouveau prédicat dont le résultat
   * est toujours la négation du résultat de l'argument.
   */
  def not[A]( p: P[A] ): P[A] = { (a:A) =>
    !p(a)
  }

  /*
   * La méthode 'and' retourne un nouveau prédicat dont le résultat
   * est toujours la conjonction des résultats des deux arguments.
   */
  def and[A]( p1: P[A], p2: P[A] ): P[A] = { (a:A) =>
    p1(a) && p2(a)
  }

  /*
   * La fonction 'or' retourne un nouveau prédicat dont le résultat
   * est toujours la disjonction des résultats des deux arguments.
   */
  def or[A]( p1: P[A], p2: P[A] ): P[A] = { (a:A) =>
    p1(a) || p2(a)
  }

  /*
   * La fonction 'exists' retourne un nouveau prédicat dont le
   * résultat est vrai si au moins un des prédicat de l'argument est
   * vrai.
   */
  def exists[A]( ps: List[P[A]] ): P[A] = { (a:A) =>
    ps match {
        case p :: Nil => p(a)
        case p :: _ if p(a) => true
        case _ :: rest => exists(rest)(a)
    }
  }

  /*
   * La fonction 'forall' retourne un nouveau prédicat dont le
   * résultat est vrai si et seulement si tous les prédicats de
   * l'argument sont vrais.
   */
  def forall[A]( ps: List[P[A]] ): P[A] = { (a:A) =>
    ps match {
        case p :: Nil => p(a)
        case p :: _ if !p(a) => false
        case _ :: rest => forall(rest)(a)
    }
  }
}
